/*
 * An object providing access to file functionality.
 *
 * Copyright (C) 2021, 2022, 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include "file_pager.h"
#include "file_provider.h"



/* Initialise the provider with a page 'mapper' for the file's contents. */

FileProvider::FileProvider(fileid_t fileid, ProviderRegistry *registry,
                           PageMapper *mapper)
: Provider(fileid, registry), _mapper(mapper)
{
}

/* Deallocate the provider's resources. */

FileProvider::~FileProvider()
{
    /* Accessors are allocated exclusively for page mappers for files, and
       so can be deleted here. */

    Accessor *accessor = _mapper->accessor();
    delete _mapper;

    accessor->close();

    if (_remove)
        accessor->remove();

    delete accessor;
}

/* Return the page mapper provided. */

PageMapper *FileProvider::mapper()
{
    return _mapper;
}

/* Return a file pager initialised with a provider, page mapper and accessor. */

long FileProvider::make_resource(flags_t flags, offset_t *size,
                                 object_flags_t *object_flags,
                                 Resource **resource)
{
    *object_flags = OBJECT_SUPPORTS_MMAP | OBJECT_HAS_SIZE;

    /* Initialise the pager with the provider and a reference to this object for
       detaching from the provider. */

    FilePager *pager = new FilePager(_fileid, this, flags, *object_flags);

    /* Obtain the size details from the pager. */

    *size = pager->get_data_size();

    this->attach();
    *resource = pager;
    return L4_EOK;
}

// vim: tabstop=4 expandtab shiftwidth=4
